﻿using fichier1.Data;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace fichier1.Model
{
    public class Partie : IObservable<Partie>
    {
        private  List<string[]> correction;
        private List<string[]> definitions;
        private List<string[]> decoupage;
        public int motCourant;
        private readonly BddContext context;
        private List<IObserver<Partie>> observers;
        public int nbMots;

        public Partie(BddContext context)
        {
            this.context = context;
            this.observers = new List<IObserver<Partie>>();
        }

        public void init(int nbMots, float complexite)
        {
            this.nbMots = nbMots;
            this.motCourant = 0;
            this.genererMots(nbMots, complexite);
            this.formaterPropositions();

            foreach (IObserver<Partie> obs in observers)
            {
                obs.OnNext(this);
            }
        }

        public void genererMots(int nb, float complexite)
        {
            correction = new List<string[]>();
            definitions = new List<string[]>();

            Random rnd = new Random();

            Prefixe[] prefixe = context.Prefixes.ToArray();
            Radical[] radical = context.Radicals.ToArray();
            Suffixe[] suffixe = context.Suffixes.ToArray();

            for(int i = 0; i < nb; i++)
            {
                string[] mot = new string[(int)complexite];
                string[] descs = new string[(int)complexite];

                if(complexite==2)
                {
                    int aleatoire = rnd.Next(0, 2);
                    if(aleatoire==0)
                    {
                        int prefixRnd = rnd.Next(0, prefixe.Length);
                        mot[0] = prefixe[prefixRnd].Pre;
                        descs[0] = prefixe[prefixRnd].Description;

                        int radicalRnd = rnd.Next(0, radical.Length);
                        mot[1] = radical[radicalRnd].Rad + radical[radicalRnd].Terminaison;
                        descs[1] = radical[radicalRnd].Description;
                    }
                    else
                    {
                        int radicalRnd = rnd.Next(0, radical.Length);
                        mot[0] = radical[radicalRnd].Rad;
                        descs[0] = radical[radicalRnd].Description;

                        int suffixRnd = rnd.Next(0, suffixe.Length);
                        mot[1] = suffixe[suffixRnd].Suf;
                        descs[1] = suffixe[suffixRnd].Description;
                    }
                }
                else
                {
                    int prefixRnd = rnd.Next(0, prefixe.Length);
                    mot[0] = prefixe[prefixRnd].Pre;
                    descs[0] = prefixe[prefixRnd].Description;

                    int radicalRnd = rnd.Next(0, radical.Length);
                    mot[1] = radical[radicalRnd].Rad;
                    descs[1] = radical[radicalRnd].Description;

                    int suffixRnd = rnd.Next(0, suffixe.Length);
                    mot[2] = suffixe[suffixRnd].Suf;
                    descs[2] = suffixe[suffixRnd].Description;
                }
                
                correction.Add(mot);
                definitions.Add(descs);
            }
        }

        public void formaterPropositions()
        {
            decoupage = new List<string[]>();
            for (int i = 0; i < correction.Count; i++)
            {
                string mot = "";
                for (int j = 0; j < correction[i].Length; j++)
                {
                    mot += correction[i][j];
                }
                string[] m = new string[1];
                m[0] = mot;
                decoupage.Add(m);
            }
        }

        public string[] getMot(int nb)
        {
            return decoupage[nb];
        }

        public void setMot(int nb, string[] mot)
        {
            decoupage[nb] = mot;
        }

        public int compterPoints()
        {
            // TODO: 
            return 0;
        }

        public string[] getCorrection(int nb)
        {
            return correction[nb];
        }

        public string[] getDefinition(int nb)
        {
            return definitions[nb];
        }

        public string[] getMotCourant()
        {
            return getMot(motCourant);
        }

        public void validerMotCourant(string[] mot)
        {
            setMot(motCourant, mot);
            motCourant++;
            if (motCourant < correction.Count)
            {
                foreach(IObserver<Partie> obs in observers)
                {
                    obs.OnNext(this);
                }
            }
            else
            {
                foreach (IObserver<Partie> obs in observers)
                {
                    obs.OnCompleted();
                }
            }
        }

        public IDisposable Subscribe(IObserver<Partie> observer)
        {
            observers.Add(observer);
            return null;
        }
    }
}
